/************************************************************************
* Copyright (c) 1995 Valentin Shopov, Blagoevgrad, Bulgaria		*
*									*
* Permission to use, copy, modify, and distribute this material for any	*
* purpose and without fee is hereby granted, provided that the above	*
* copyright notice and this permission notice appear in all copies, and	*
* that the name of the author not be used in advertising or publicity 	*
* pertaining to this material without the specific, prior written 	*
* permission of the author.						*
*									*
* THE AUTHOR MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY	*  
* OF THIS MATERIAL FOR ANY PURPOSE. IT IS PROVIDED "AS IS", WITHOUT ANY	* 
* EXPRESS OR IMPLIED WARRANTIES.					*
*************************************************************************/

/**************************ciilib1.2.2***********************************
 *                                                                      *
 *        Character to Inline Image Conversion Library (CIILIB)         *
 *                                                                      *
 *     		8x16 BDF Font to CIILIB XBM source convertor.		*
 *              	by Valentin Shopov		                *
 *			valentin@rs.aubg.bg				*
 ************************************************************************/





#include <stdio.h>
#include <string.h>

char curcol[2048], currow[2048];
int CHIND[256][256], width[256], height[256];

char *
getcol (row, numcol)
     char *row;
     int numcol;
{
  int i, j, k;
  char *tmpcol;
  if (numcol <= 0)
    return row;
  tmpcol = curcol;
  for (i = 0, j = 0, k = 0; row[i] != '\n'; i++)
    {
      if (row[i] == ' ' || row[i] == '\t')
	{
	  if (j)
	    {
	      tmpcol[j] = '\0';
	      j = 0;
	      if (++k == numcol)
		return tmpcol;
	    }
	  continue;
	}
      tmpcol[j++] = row[i];
    }
  if (k == numcol - 1)
    {
      tmpcol[j] = '\0';
    }
  else
    tmpcol[0] = '\0';
  return tmpcol;
}

mirror (src)
     char *src;
{
  int bmrow, i, lbit = 0x0001, hbit = 0x0080, dst;
  sscanf (src, "%02x", &bmrow);
  for (i = 0, dst = 0; i < 8; i++)
    {
      dst |= (lbit & bmrow) ? hbit : 0;
      lbit <<= 1;
      hbit >>= 1;
    }
  return dst;
}
static char sname[50];

main (ac, av)
     char *av[];
{
  int XBMPOS = 0, NOFCH, NOFB, CODE, BMB = 0, cnvtb, x, i, numof256c = 0;
  char *row;
  FILE *dstf, *srcf;
  if (ac < 3 && (ac - 1) % 2)
    {
      fprintf (stderr, "Usage: %s  <8x16-bdf-file-name(s)> <charset abbreviation(s)>\n", av[0]);
      exit (1);
    }

  row = currow;
  sprintf (sname, "_xbmraw_256.c\0");


  printf ("Creating \"%s\"...\n", sname);
  dstf = fopen (sname, "w+");
  if (dstf == NULL)
    {
      fprintf (stderr, "Error opening \"%s\"!", sname);
      exit (1);
    }
/*  fprintf (dstf, "char *xbmsrc_256[%d][257] = {\n", (ac + 1) / 2);*/
  fprintf (dstf, "char *xbmsrc_256[][257] = {\n");

  for (i = 1; i < (ac + 1) / 2; i++)
    {
      if (av[i][0] == 0)
	continue;
      numof256c++;
      if (!XBMPOS)
	fprintf (dstf, "{");
      for (x = 0; x < 256; x++)
	{
	  CHIND[i - 1][x] = -1;
	}
      srcf = fopen (av[i], "r");
      if (srcf == NULL)
	{
	  fprintf (stderr, "Error opening \"%s\"!", av[i]);
	  exit (1);
	}
      for (; fgets (row, 2048, srcf) != NULL;)
	{
	  if (!XBMPOS)
	    {

	      if (!strcmp ("FONTBOUNDINGBOX", getcol (row, 1)))
		{
		  int w, h;
		  w = atoi (getcol (row, 2));
		  h = atoi (getcol (row, 3));
		  if (! (w*2==h && (w==7 || w==8)))
		    {
		      printf ("%s BDF font is not 7x14 or 8x16 size !!!\n", av[i]);
		      exit (1);
		    }
		  width[i-1] = w;
		  height[i-1] = h;
		}

	      if (!strcmp ("CHARS", getcol (row, 1)))
		{
		  NOFCH = atoi (getcol (row, 2));
		  if (NOFCH > 256)
		    {
		      fprintf (stderr, "Number of chars in %s must be <= 256 !", av[i]);
		      exit (1);
		    }

		}

	    }

	  if (!strcmp ("ENCODING", getcol (row, 1)))
	    {
	      NOFB = 0;
	      CODE = atoi (getcol (row, 2));
	      fprintf (dstf, "/* %02x CODE */\n\"\\\n", CODE);
	      CHIND[i - 1][CODE] = XBMPOS++;
	    }

	  if (!strcmp ("ENDCHAR", getcol (row, 1)))
	    {
	      BMB = 0;
	      fprintf (dstf, "\",\n");
	      printf ("Completed...  %d%%\r", (XBMPOS * 100) / NOFCH);
	    }


	  if (BMB)
	    {
	      if (NOFB)
		{
		  fprintf (dstf, "\\\n");
		}
	      cnvtb = mirror (getcol (row, 1));
	      fprintf (dstf, "\\%03o", cnvtb);
	      NOFB = 1;
	    }

	  if (!strcmp ("BITMAP", getcol (row, 1)))
	    BMB = 1;

	}
      fclose (srcf);
      if (NOFCH != XBMPOS)
	{
	  printf ("\nSomething wrong! Please check your %s BDF file!\n", av[i]);
	  exit (1);
	}
      printf ("\nNumber of defined chars: %d\nNumber of processed chars: %d\nDone with processing \"%s\".\n", NOFCH, XBMPOS, av[i]);

      XBMPOS = BMB = 0;
      fprintf (dstf, "0},\n");
    }
  fprintf (dstf, "0};\n");
  fprintf (dstf, "int xbmind_256[][257] = {\n");
  for (i = 0; i < (ac - 1) / 2; i++)
    {
      if (av[i+1][0] == 0)
	continue;
      fprintf (dstf, "{");
      for (x = 0; x < 256; x++)
	{
	  fprintf (dstf, "%d,", CHIND[i][x]);
	  if (x % 10 == 9)
	    fprintf (dstf, "\n");
	}
      fprintf (dstf, "0},\n");
    }
  fprintf (dstf, "0};\n");
  fprintf (dstf, "int numof256c=%d;\n", numof256c);
  fprintf (dstf, "char *chsetabbr[]={");
  for (i = (ac + 1) / 2; av[i]; i++)
    {
      if (av[i - (ac-1)/2][0])
	fprintf (dstf, "\"%s\",", av[i]);
    }
  fprintf (dstf, "0};\n");
  fprintf (dstf, "int xbm256w[]={");
  for (i = 0; i < (ac - 1) / 2; i++)
    {
      if (av[i+1][0])
	fprintf(dstf, "%d,", width[i]);
    }
  fprintf(dstf, "0};\n");
  fprintf (dstf, "int xbm256h[]={");
  for (i = 0; i < (ac - 1) / 2; i++)
    {
      if (av[i+1][0])
	fprintf(dstf, "%d,", height[i]);
    }
  fprintf(dstf, "0};\n");
  fclose (dstf);
  printf ("\nDone with creating \"_xbmraw_256.c\".\n");
  exit (0);
}
